@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack import _log
+from openstack.baremetal.v1 import _common
 from openstack.baremetal.v1 import chassis as _chassis
 from openstack.baremetal.v1 import driver as _driver
 from openstack.baremetal.v1 import node as _node
@@ -19,8 +21,13 @@ from openstack import proxy
 from openstack import utils
 
 
+_logger = _log.setup_logging('openstack')
+
+
 class Proxy(proxy.Proxy):
 
+    retriable_status_codes = _common.RETRIABLE_STATUS_CODES
+
     def chassis(self, details=False, **query):
         """Retrieve a generator of chassis.
 
@@ -125,12 +132,19 @@ class Proxy(proxy.Proxy):
         return self._delete(_chassis.Chassis, chassis,
                             ignore_missing=ignore_missing)
 
-    def drivers(self):
+    def drivers(self, details=False):
         """Retrieve a generator of drivers.
 
+        :param bool details: A boolean indicating whether the detailed
+            information for every driver should be returned.
         :returns: A generator of driver instances.
         """
-        return self._list(_driver.Driver, paginated=False)
+        kwargs = {}
+        # NOTE(dtantsur): details are available starting with API microversion
+        # 1.30. Thus we do not send any value if not needed.
+        if details:
+            kwargs['details'] = True
+        return self._list(_driver.Driver, paginated=False, **kwargs)
 
     def get_driver(self, driver):
         """Get a specific driver.
@@ -227,18 +241,147 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_node.Node, node)
 
-    def update_node(self, node, **attrs):
+    def update_node(self, node, retry_on_conflict=True, **attrs):
         """Update a node.
 
         :param chassis: Either the name or the ID of a node or an instance
             of :class:`~openstack.baremetal.v1.node.Node`.
+        :param bool retry_on_conflict: Whether to retry HTTP CONFLICT error.
+            Most of the time it can be retried, since it is caused by the node
+            being locked. However, when setting ``instance_id``, this is
+            a normal code and should not be retried.
         :param dict attrs: The attributes to update on the node represented
             by the ``node`` parameter.
 
         :returns: The updated node.
         :rtype: :class:`~openstack.baremetal.v1.node.Node`
         """
-        return self._update(_node.Node, node, **attrs)
+        res = self._get_resource(_node.Node, node, **attrs)
+        return res.commit(self, retry_on_conflict=retry_on_conflict)
+
+    def set_node_provision_state(self, node, target, config_drive=None,
+                                 clean_steps=None, rescue_password=None,
+                                 wait=False, timeout=None):
+        """Run an action modifying node's provision state.
+
+        This call is asynchronous, it will return success as soon as the Bare
+        Metal service acknowledges the request.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param target: Provisioning action, e.g. ``active``, ``provide``.
+            See the Bare Metal service documentation for available actions.
+        :param config_drive: Config drive to pass to the node, only valid
+            for ``active` and ``rebuild`` targets.
+        :param clean_steps: Clean steps to execute, only valid for ``clean``
+            target.
+        :param rescue_password: Password for the rescue operation, only valid
+            for ``rescue`` target.
+        :param wait: Whether to wait for the node to get into the expected
+            state. The expected state is determined from a combination of
+            the current provision state and ``target``.
+        :param timeout: If ``wait`` is set to ``True``, specifies how much (in
+            seconds) to wait for the expected state to be reached. The value of
+            ``None`` (the default) means no client-side timeout.
+
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        :raises: ValueError if ``config_drive``, ``clean_steps`` or
+            ``rescue_password`` are provided with an invalid ``target``.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.set_provision_state(self, target, config_drive=config_drive,
+                                       clean_steps=clean_steps,
+                                       rescue_password=rescue_password,
+                                       wait=wait, timeout=timeout)
+
+    def wait_for_nodes_provision_state(self, nodes, expected_state,
+                                       timeout=None,
+                                       abort_on_failed_state=True):
+        """Wait for the nodes to reach the expected state.
+
+        :param nodes: List of nodes - name, ID or
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param expected_state: The expected provisioning state to reach.
+        :param timeout: If ``wait`` is set to ``True``, specifies how much (in
+            seconds) to wait for the expected state to be reached. The value of
+            ``None`` (the default) means no client-side timeout.
+        :param abort_on_failed_state: If ``True`` (the default), abort waiting
+            if any node reaches a failure state which does not match the
+            expected one. Note that the failure state for ``enroll`` ->
+            ``manageable`` transition is ``enroll`` again.
+
+        :return: The list of :class:`~openstack.baremetal.v1.node.Node`
+            instances that reached the requested state.
+        """
+        log_nodes = ', '.join(n.id if isinstance(n, _node.Node) else n
+                              for n in nodes)
+
+        finished = []
+        remaining = nodes
+        for count in utils.iterate_timeout(
+                timeout,
+                "Timeout waiting for nodes %(nodes)s to reach "
+                "target state '%(state)s'" % {'nodes': log_nodes,
+                                              'state': expected_state}):
+            nodes = [self.get_node(n) for n in remaining]
+            remaining = []
+            for n in nodes:
+                if n._check_state_reached(self, expected_state,
+                                          abort_on_failed_state):
+                    finished.append(n)
+                else:
+                    remaining.append(n)
+
+            if not remaining:
+                return finished
+
+            _logger.debug('Still waiting for nodes %(nodes)s to reach state '
+                          '"%(target)s"',
+                          {'nodes': ', '.join(n.id for n in remaining),
+                           'target': expected_state})
+
+    def wait_for_node_reservation(self, node, timeout=None):
+        """Wait for a lock on the node to be released.
+
+        Bare metal nodes in ironic have a reservation lock that
+        is used to represent that a conductor has locked the node
+        while performing some sort of action, such as changing
+        configuration as a result of a machine state change.
+
+        This lock can occur during power syncronization, and prevents
+        updates to objects attached to the node, such as ports.
+
+        Note that nothing prevents a conductor from acquiring the lock again
+        after this call returns, so it should be treated as best effort.
+
+        Returns immediately if there is no reservation on the node.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param timeout: How much (in seconds) to wait for the lock to be
+            released. The value of ``None`` (the default) means no timeout.
+
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.wait_for_reservation(self, timeout=timeout)
+
+    def validate_node(self, node, required=('boot', 'deploy', 'power')):
+        """Validate required information on a node.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param required: List of interfaces that are required to pass
+            validation. The default value is the list of minimum required
+            interfaces for provisioning.
+
+        :return: dict mapping interface names to
+            :class:`~openstack.baremetal.v1.node.ValidationResult` objects.
+        :raises: :exc:`~openstack.exceptions.ValidationException` if validation
+            fails for a required interface.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.validate(self, required=required)
 
     def delete_node(self, node, ignore_missing=True):
         """Delete a node.
@@ -375,47 +518,6 @@ class Proxy(proxy.Proxy):
         """
         return self._delete(_port.Port, port, ignore_missing=ignore_missing)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use port_groups instead")
-    def portgroups(self, details=False, **query):
-        """Retrieve a generator of port groups.
-
-        :param details: A boolean indicating whether the detailed information
-                        for every portgroup should be returned.
-        :param dict query: Optional query parameters to be sent to restrict
-            the portgroups returned. Available parameters include:
-
-            * ``address``: Only return portgroups with the specified physical
-                    hardware address, typically a MAC address.
-            * ``fields``: A list containing one or more fields to be returned
-                    in the response. This may lead to some performance gain
-                    because other fields of the resource are not refreshed.
-            * ``limit``: Requests at most the specified number of portgroups
-                    returned from the query.
-            * ``marker``: Specifies the ID of the last-seen portgroup. Use the
-                    ``limit`` parameter to make an initial limited request and
-                    use the ID of the last-seen portgroup from the response as
-                    the ``marker`` value in a subsequent limited request.
-            * ``node``:only return the ones associated with this specific node
-                    (name or UUID), or an empty set if not found.
-            * ``sort_dir``: Sorts the response by the requested sort direction.
-                    A valid value is ``asc`` (ascending) or ``desc``
-                    (descending). Default is ``asc``. You can specify multiple
-                    pairs of sort key and sort direction query parameters. If
-                    you omit the sort direction in a pair, the API uses the
-                    natural sorting direction of the server attribute that is
-                    provided as the ``sort_key``.
-            * ``sort_key``: Sorts the response by the this attribute value.
-                    Default is ``id``. You can specify multiple pairs of sort
-                    key and sort direction query parameters. If you omit the
-                    sort direction in a pair, the API uses the natural sorting
-                    direction of the server attribute that is provided as the
-                    ``sort_key``.
-
-        :returns: A generator of portgroup instances.
-        """
-        return self.port_groups(details=details, **query)
-
     def port_groups(self, details=False, **query):
         """Retrieve a generator of port groups.
 
@@ -456,20 +558,6 @@ class Proxy(proxy.Proxy):
         cls = _portgroup.PortGroupDetail if details else _portgroup.PortGroup
         return self._list(cls, paginated=True, **query)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use create_port_group instead")
-    def create_portgroup(self, **attrs):
-        """Create a new port group from attributes.
-
-        :param dict attrs: Keyword arguments that will be used to create a
-             :class:`~openstack.baremetal.v1.port_group.PortGroup`, it
-             comprises of the properties on the ``PortGroup`` class.
-
-        :returns: The results of portgroup creation.
-        :rtype: :class:`~openstack.baremetal.v1.port_group.PortGroup`.
-        """
-        return self.create_port_group(**attrs)
-
     def create_port_group(self, **attrs):
         """Create a new portgroup from attributes.
 
@@ -482,21 +570,6 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_portgroup.PortGroup, **attrs)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use find_port_group instead")
-    def find_portgroup(self, name_or_id, ignore_missing=True):
-        """Find a single port group.
-
-        :param str name_or_id: The name or ID of a portgroup.
-        :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
-            when the port group does not exist.  When set to `True``, None will
-            be returned when attempting to find a nonexistent port group.
-        :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
-            object or None.
-        """
-        return self.find_port_group(name_or_id, ignore_missing=ignore_missing)
-
     def find_port_group(self, name_or_id, ignore_missing=True):
         """Find a single port group.
 
@@ -511,26 +584,6 @@ class Proxy(proxy.Proxy):
         return self._find(_portgroup.PortGroup, name_or_id,
                           ignore_missing=ignore_missing)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use get_port_group instead")
-    def get_portgroup(self, portgroup, **query):
-        """Get a specific port group.
-
-        :param portgroup: The value can be the name or ID of a chassis or a
-            :class:`~openstack.baremetal.v1.port_group.PortGroup` instance.
-        :param dict query: Optional query parameters to be sent to restrict
-            the portgroup properties returned. Available parameters include:
-
-            * ``fields``: A list containing one or more fields to be returned
-                    in the response. This may lead to some performance gain
-                    because other fields of the resource are not refreshed.
-
-        :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
-            port group matching the name or ID could be found.
-        """
-        return self.get_port_group(portgroup, **query)
-
     def get_port_group(self, port_group, **query):
         """Get a specific port group.
 
@@ -549,22 +602,6 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_portgroup.PortGroup, port_group, **query)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use update_port_group instead")
-    def update_portgroup(self, portgroup, **attrs):
-        """Update a port group.
-
-        :param chassis: Either the name or the ID of a port group or
-            an instance of
-            :class:`~openstack.baremetal.v1.port_group.PortGroup`.
-        :param dict attrs: The attributes to update on the port group
-            represented by the ``portgroup`` parameter.
-
-        :returns: The updated port group.
-        :rtype: :class:`~openstack.baremetal.v1.port_group.PortGroup`
-        """
-        return self.update_port_group(portgroup, **attrs)
-
     def update_port_group(self, port_group, **attrs):
         """Update a port group.
 
@@ -579,26 +616,6 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_portgroup.PortGroup, port_group, **attrs)
 
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details="Use delete_port_group instead")
-    def delete_portgroup(self, portgroup, ignore_missing=True):
-        """Delete a port group.
-
-        :param portgroup: The value can be either the name or ID of a port
-            group or a
-            :class:`~openstack.baremetal.v1.port_group.PortGroup`
-            instance.
-        :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
-            when the port group could not be found. When set to ``True``, no
-            exception will be raised when attempting to delete a non-existent
-            port group.
-
-        :returns: The instance of the port group which was deleted.
-        :rtype: :class:`~openstack.baremetal.v1.port_group.PortGroup`.
-        """
-        return self.delete_port_group(portgroup, ignore_missing=ignore_missing)
-
     def delete_port_group(self, port_group, ignore_missing=True):
         """Delete a port group.
 
@@ -617,3 +634,58 @@ class Proxy(proxy.Proxy):
         """
         return self._delete(_portgroup.PortGroup, port_group,
                             ignore_missing=ignore_missing)
+
+    def attach_vif_to_node(self, node, vif_id):
+        """Attach a VIF to the node.
+
+        The exact form of the VIF ID depends on the network interface used by
+        the node. In the most common case it is a Network service port
+        (NOT a Bare Metal port) ID. A VIF can only be attached to one node
+        at a time.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param string vif_id: Backend-specific VIF ID.
+        :return: ``None``
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
+            does not support the VIF API.
+        """
+        res = self._get_resource(_node.Node, node)
+        res.attach_vif(self, vif_id)
+
+    def detach_vif_from_node(self, node, vif_id, ignore_missing=True):
+        """Detach a VIF from the node.
+
+        The exact form of the VIF ID depends on the network interface used by
+        the node. In the most common case it is a Network service port
+        (NOT a Bare Metal port) ID.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param string vif_id: Backend-specific VIF ID.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the VIF does not exist. Otherwise, ``False``
+                    is returned.
+        :return: ``True`` if the VIF was detached, otherwise ``False``.
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
+            does not support the VIF API.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.detach_vif(self, vif_id, ignore_missing=ignore_missing)
+
+    def list_node_vifs(self, node):
+        """List IDs of VIFs attached to the node.
+
+        The exact form of the VIF ID depends on the network interface used by
+        the node. In the most common case it is a Network service port
+        (NOT a Bare Metal port) ID.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :return: List of VIF IDs as strings.
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
+            does not support the VIF API.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.list_vifs(self)
