@@ -16,6 +16,7 @@
 Exception definitions.
 """
 
+import json
 import re
 
 from requests import exceptions as _rex
@@ -108,11 +109,6 @@ class HttpException(SDKException, _rex.HTTPError):
         return self.__unicode__()
 
 
-class NotFoundException(HttpException):
-    """HTTP 404 Not Found."""
-    pass
-
-
 class BadRequestException(HttpException):
     """HTTP 400 Bad Request."""
     pass
@@ -142,11 +138,14 @@ class DuplicateResource(SDKException):
     pass
 
 
-class ResourceNotFound(NotFoundException):
+class ResourceNotFound(HttpException):
     """No resource exists with that name or id."""
     pass
 
 
+NotFoundException = ResourceNotFound
+
+
 class ResourceTimeout(SDKException):
     """Timeout waiting for resource."""
     pass
@@ -162,6 +161,24 @@ class InvalidResourceQuery(SDKException):
     pass
 
 
+def _extract_message(obj):
+    if isinstance(obj, dict):
+        # Most of services: compute, network
+        if obj.get('message'):
+            return obj['message']
+        # Ironic starting with Stein
+        elif obj.get('faultstring'):
+            return obj['faultstring']
+    elif isinstance(obj, six.string_types):
+        # Ironic before Stein has double JSON encoding, nobody remembers why.
+        try:
+            obj = json.loads(obj)
+        except Exception:
+            pass
+        else:
+            return _extract_message(obj)
+
+
 def raise_from_response(response, error_message=None):
     """Raise an instance of an HTTPException based on keystoneauth response."""
     if response.status_code < 400:
@@ -185,8 +202,7 @@ def raise_from_response(response, error_message=None):
 
         try:
             content = response.json()
-            messages = [obj.get('message') for obj in content.values()
-                        if isinstance(obj, dict)]
+            messages = [_extract_message(obj) for obj in content.values()]
             # Join all of the messages together nicely and filter out any
             # objects that don't have a "message" attr.
             details = '\n'.join(msg for msg in messages if msg)
@@ -199,10 +215,9 @@ def raise_from_response(response, error_message=None):
         details = list(set([msg for msg in details if msg]))
         # Return joined string separated by colons.
         details = ': '.join(details)
-    if not details and response.reason:
-        details = response.reason
-    else:
-        details = response.text
+
+    if not details:
+        details = response.reason if response.reason else response.text
 
     http_status = response.status_code
     request_id = response.headers.get('x-openstack-request-id')
@@ -213,6 +228,10 @@ def raise_from_response(response, error_message=None):
     )
 
 
+class UnsupportedServiceVersion(Warning):
+    """The user has configured a major version that SDK doesn't know."""
+
+
 class ArgumentDeprecationWarning(Warning):
     """A deprecated argument has been provided."""
     pass
@@ -220,3 +239,15 @@ class ArgumentDeprecationWarning(Warning):
 
 class ConfigException(SDKException):
     """Something went wrong with parsing your OpenStack Config."""
+
+
+class NotSupported(SDKException):
+    """Request cannot be performed by any supported API version."""
+
+
+class ValidationException(SDKException):
+    """Validation failed for resource."""
+
+
+class TaskManagerStopped(SDKException):
+    """Operations were attempted on a stopped TaskManager."""
