@@ -15,9 +15,11 @@ from openstack.identity.v3 import credential as _credential
 from openstack.identity.v3 import domain as _domain
 from openstack.identity.v3 import endpoint as _endpoint
 from openstack.identity.v3 import group as _group
+from openstack.identity.v3 import limit as _limit
 from openstack.identity.v3 import policy as _policy
 from openstack.identity.v3 import project as _project
 from openstack.identity.v3 import region as _region
+from openstack.identity.v3 import registered_limit as _registered_limit
 from openstack.identity.v3 import role as _role
 from openstack.identity.v3 import role_assignment as _role_assignment
 from openstack.identity.v3 import role_domain_group_assignment \
@@ -31,10 +33,10 @@ from openstack.identity.v3 import role_project_user_assignment \
 from openstack.identity.v3 import service as _service
 from openstack.identity.v3 import trust as _trust
 from openstack.identity.v3 import user as _user
-from openstack import proxy2 as proxy
+from openstack import proxy
 
 
-class Proxy(proxy.BaseProxy):
+class Proxy(proxy.Proxy):
 
     def create_credential(self, **attrs):
         """Create a new credential from attributes
@@ -459,7 +461,7 @@ class Proxy(proxy.BaseProxy):
         """
         self._delete(_project.Project, project, ignore_missing=ignore_missing)
 
-    def find_project(self, name_or_id, ignore_missing=True):
+    def find_project(self, name_or_id, ignore_missing=True, **attrs):
         """Find a single project
 
         :param name_or_id: The name or ID of a project.
@@ -471,7 +473,7 @@ class Proxy(proxy.BaseProxy):
         :returns: One :class:`~openstack.identity.v3.project.Project` or None
         """
         return self._find(_project.Project, name_or_id,
-                          ignore_missing=ignore_missing)
+                          ignore_missing=ignore_missing, **attrs)
 
     def get_project(self, project):
         """Get a single project
@@ -497,6 +499,22 @@ class Proxy(proxy.BaseProxy):
         # TODO(briancurtin): This is paginated but requires base list changes.
         return self._list(_project.Project, paginated=False, **query)
 
+    def user_projects(self, user, **query):
+        """Retrieve a generator of projects to which the user has authorization
+           to access.
+
+        :param user: Either the user id or an instance of
+                     :class:`~openstack.identity.v3.user.User`
+        :param kwargs \*\*query: Optional query parameters to be sent to limit
+                                 the resources being returned.
+
+        :returns: A generator of project instances.
+        :rtype: :class:`~openstack.identity.v3.project.UserProject`
+        """
+        user = self._get_resource(_user.User, user)
+        return self._list(_project.UserProject, paginated=True,
+                          user_id=user.id, **query)
+
     def update_project(self, project, **attrs):
         """Update a project
 
@@ -615,7 +633,7 @@ class Proxy(proxy.BaseProxy):
         """
         self._delete(_user.User, user, ignore_missing=ignore_missing)
 
-    def find_user(self, name_or_id, ignore_missing=True):
+    def find_user(self, name_or_id, ignore_missing=True, **attrs):
         """Find a single user
 
         :param name_or_id: The name or ID of a user.
@@ -627,7 +645,7 @@ class Proxy(proxy.BaseProxy):
         :returns: One :class:`~openstack.identity.v3.user.User` or None
         """
         return self._find(_user.User, name_or_id,
-                          ignore_missing=ignore_missing)
+                          ignore_missing=ignore_missing, **attrs)
 
     def get_user(self, user):
         """Get a single user
@@ -956,3 +974,143 @@ class Proxy(proxy.BaseProxy):
         """
         return self._list(_role_assignment.RoleAssignment,
                           paginated=False, **query)
+
+    def registered_limits(self, **query):
+        """Retrieve a generator of registered_limits
+
+        :param kwargs \*\*query: Optional query parameters to be sent to limit
+            the registered_limits being returned.
+
+        :returns: A generator of registered_limits instances.
+        :rtype: :class:
+            `~openstack.identity.v3.registered_limit.RegisteredLimit`
+        """
+        return self._list(_registered_limit.RegisteredLimit, paginated=False,
+                          **query)
+
+    def get_registered_limit(self, registered_limit):
+        """Get a single registered_limit
+
+        :param registered_limit: The value can be the ID of a registered_limit
+            or a :class:
+            `~openstack.identity.v3.registered_limit.RegisteredLimit` instance.
+
+        :returns: One :class:
+                  `~openstack.identity.v3.registered_limit.RegisteredLimit`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_registered_limit.RegisteredLimit, registered_limit)
+
+    def create_registered_limit(self, **attrs):
+        """Create a new registered_limit from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.identity.v3.registered_limit.RegisteredLimit`,
+            comprised of the properties on the RegisteredLimit class.
+
+        :returns: The results of registered_limit creation.
+        :rtype: :class:
+            `~openstack.identity.v3.registered_limit.RegisteredLimit`
+        """
+        return self._create(_registered_limit.RegisteredLimit, **attrs)
+
+    def update_registered_limit(self, registered_limit, **attrs):
+        """Update a registered_limit
+
+        :param registered_limit: Either the ID of a registered_limit. or a
+            :class:`~openstack.identity.v3.registered_limit.RegisteredLimit`
+            instance.
+        :param dict kwargs: The attributes to update on the registered_limit
+            represented by ``value``.
+
+        :returns: The updated registered_limit.
+        :rtype: :class:
+            `~openstack.identity.v3.registered_limit.RegisteredLimit`
+        """
+        return self._update(_registered_limit.RegisteredLimit,
+                            registered_limit, **attrs)
+
+    def delete_registered_limit(self, registered_limit, ignore_missing=True):
+        """Delete a registered_limit
+
+        :param registered_limit: The value can be either the ID of a
+            registered_limit or a
+            :class:`~openstack.identity.v3.registered_limit.RegisteredLimit`
+            instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the registered_limit does not exist. When set to ``True``, no
+            exception will be thrown when attempting to delete a nonexistent
+            registered_limit.
+
+        :returns: ``None``
+        """
+        self._delete(_registered_limit.RegisteredLimit, registered_limit,
+                     ignore_missing=ignore_missing)
+
+    def limits(self, **query):
+        """Retrieve a generator of limits
+
+        :param kwargs \*\*query: Optional query parameters to be sent to limit
+            the limits being returned.
+
+        :returns: A generator of limits instances.
+        :rtype: :class:`~openstack.identity.v3.limit.Limit`
+        """
+        return self._list(_limit.Limit, paginated=False,
+                          **query)
+
+    def get_limit(self, limit):
+        """Get a single limit
+
+        :param limit: The value can be the ID of a limit
+            or a :class:`~openstack.identity.v3.limit.Limit` instance.
+
+        :returns: One :class:
+                  `~openstack.identity.v3.limit.Limit`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        return self._get(_limit.Limit, limit)
+
+    def create_limit(self, **attrs):
+        """Create a new limit from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.identity.v3.limit.Limit`, comprised of the
+            properties on the Limit class.
+
+        :returns: The results of limit creation.
+        :rtype: :class:`~openstack.identity.v3.limit.Limit`
+        """
+        return self._create(_limit.Limit, **attrs)
+
+    def update_limit(self, limit, **attrs):
+        """Update a limit
+
+        :param limit: Either the ID of a limit. or a
+            :class:`~openstack.identity.v3.limit.Limit` instance.
+        :param dict kwargs: The attributes to update on the limit represented
+            by ``value``.
+
+        :returns: The updated limit.
+        :rtype: :class:`~openstack.identity.v3.limit.Limit`
+        """
+        return self._update(_limit.Limit,
+                            limit, **attrs)
+
+    def delete_limit(self, limit, ignore_missing=True):
+        """Delete a limit
+
+        :param limit: The value can be either the ID of a limit or a
+            :class:`~openstack.identity.v3.limit.Limit` instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the limit does not exist. When set to ``True``, no exception will
+            be thrown when attempting to delete a nonexistent limit.
+
+        :returns: ``None``
+        """
+        self._delete(limit.Limit, limit,
+                     ignore_missing=ignore_missing)
