@@ -12,48 +12,46 @@
 
 import uuid
 
-from openstack.message import message_service
-from openstack import resource2
+from openstack import resource
 
 
-class Message(resource2.Resource):
+class Message(resource.Resource):
     # FIXME(anyone): The name string of `location` field of Zaqar API response
     # is lower case. That is inconsistent with the guide from API-WG. This is
     # a workaround for this issue.
-    location = resource2.Header("location")
+    location = resource.Header("location")
 
     resources_key = 'messages'
     base_path = '/queues/%(queue_name)s/messages'
-    service = message_service.MessageService()
 
     # capabilities
     allow_create = True
     allow_list = True
-    allow_get = True
+    allow_fetch = True
     allow_delete = True
 
-    _query_mapping = resource2.QueryParameters("echo", "include_claimed")
+    _query_mapping = resource.QueryParameters("echo", "include_claimed")
 
     # Properties
     #: The value in second to specify how long the message has been
     #: posted to the queue.
-    age = resource2.Body("age")
+    age = resource.Body("age")
     #: A dictionary specifies an arbitrary document that constitutes the
     #: body of the message being sent.
-    body = resource2.Body("body")
+    body = resource.Body("body")
     #: An uri string describe the location of the message resource.
-    href = resource2.Body("href")
+    href = resource.Body("href")
     #: The value in seconds to specify how long the server waits before
     #: marking the message as expired and removing it from the queue.
-    ttl = resource2.Body("ttl")
+    ttl = resource.Body("ttl")
     #: The name of target queue message is post to or got from.
-    queue_name = resource2.URI("queue_name")
+    queue_name = resource.URI("queue_name")
     #: The ID to identify the client accessing Zaqar API. Must be specified
     #: in header for each API request.
-    client_id = resource2.Header("Client-ID")
+    client_id = resource.Header("Client-ID")
     #: The ID to identify the project accessing Zaqar API. Must be specified
     #: in case keystone auth is not enabled in Zaqar service.
-    project_id = resource2.Header("X-PROJECT-ID")
+    project_id = resource.Header("X-PROJECT-ID")
 
     def post(self, session, messages):
         request = self._prepare_request(requires_id=False, prepend_key=True)
@@ -63,7 +61,7 @@ class Message(resource2.Resource):
         }
         request.headers.update(headers)
         request.body = {'messages': messages}
-        response = session.post(request.uri, endpoint_filter=self.service,
+        response = session.post(request.url,
                                 json=request.body, headers=request.headers)
 
         return response.json()['resources']
@@ -72,7 +70,7 @@ class Message(resource2.Resource):
     def list(cls, session, paginated=True, **params):
         """This method is a generator which yields message objects.
 
-        This is almost the copy of list method of resource2.Resource class.
+        This is almost the copy of list method of resource.Resource class.
         The only difference is the request header now includes `Client-ID`
         and `X-PROJECT-ID` fields which are required by Zaqar v2 API.
         """
@@ -86,7 +84,7 @@ class Message(resource2.Resource):
 
         query_params = cls._query_mapping._transpose(params)
         while more_data:
-            resp = session.get(uri, endpoint_filter=cls.service,
+            resp = session.get(uri,
                                headers=headers, params=query_params)
             resp = resp.json()
             resp = resp[cls.resources_key]
@@ -109,7 +107,7 @@ class Message(resource2.Resource):
             query_params["limit"] = yielded
             query_params["marker"] = new_marker
 
-    def get(self, session, requires_id=True):
+    def fetch(self, session, requires_id=True, error_message=None):
         request = self._prepare_request(requires_id=requires_id)
         headers = {
             "Client-ID": self.client_id or str(uuid.uuid4()),
@@ -117,7 +115,7 @@ class Message(resource2.Resource):
         }
 
         request.headers.update(headers)
-        response = session.get(request.uri, endpoint_filter=self.service,
+        response = session.get(request.url,
                                headers=headers)
         self._translate_response(response)
 
@@ -135,8 +133,8 @@ class Message(resource2.Resource):
         # parameter when deleting a message that has been claimed, we
         # rebuild the request URI if claim_id is not None.
         if self.claim_id:
-            request.uri += '?claim_id=%s' % self.claim_id
-        response = session.delete(request.uri, endpoint_filter=self.service,
+            request.url += '?claim_id=%s' % self.claim_id
+        response = session.delete(request.url,
                                   headers=headers)
 
         self._translate_response(response, has_body=False)
