@@ -10,26 +10,26 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
-from openstack import resource2 as resource
+from openstack.network.v2 import tag
+from openstack import resource
 
 
-class SecurityGroup(resource.Resource):
+class SecurityGroup(resource.Resource, tag.TagMixin):
     resource_key = 'security_group'
     resources_key = 'security_groups'
     base_path = '/security-groups'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
 
     _query_mapping = resource.QueryParameters(
         'description', 'name',
         project_id='tenant_id',
+        **tag.TagMixin._tag_query_parameters
     )
 
     # Properties
