@@ -19,15 +19,15 @@ from openstack.orchestration.v1 import stack_environment as _stack_environment
 from openstack.orchestration.v1 import stack_files as _stack_files
 from openstack.orchestration.v1 import stack_template as _stack_template
 from openstack.orchestration.v1 import template as _template
-from openstack import proxy2
+from openstack import proxy
 
 
-class Proxy(proxy2.BaseProxy):
+class Proxy(proxy.Proxy):
 
     def create_stack(self, preview=False, **attrs):
         """Create a new stack from attributes
 
-        :param bool perview: When ``True``, returns
+        :param bool preview: When ``True``, returns
             an :class:`~openstack.orchestration.v1.stack.StackPreview` object,
             otherwise an :class:`~openstack.orchestration.v1.stack.Stack`
             object.
@@ -125,7 +125,7 @@ class Proxy(proxy2.BaseProxy):
         else:
             stk_obj = _stack.Stack.existing(id=stack)
 
-        stk_obj.check(self._session)
+        stk_obj.check(self)
 
     def get_stack_template(self, stack):
         """Get template used by a stack
@@ -184,7 +184,7 @@ class Proxy(proxy2.BaseProxy):
             stk = self._find(_stack.Stack, stack, ignore_missing=False)
 
         obj = _stack_files.StackFiles(stack_name=stk.name, stack_id=stk.id)
-        return obj.get(self._session)
+        return obj.fetch(self)
 
     def resources(self, stack, **query):
         """Return a generator of resources
@@ -357,6 +357,6 @@ class Proxy(proxy2.BaseProxy):
                 "'template_url' must be specified when template is None")
 
         tmpl = _template.Template.new()
-        return tmpl.validate(self._session, template, environment=environment,
+        return tmpl.validate(self, template, environment=environment,
                              template_url=template_url,
                              ignore_errors=ignore_errors)
