@@ -124,6 +124,8 @@ class TestCase(base.TestCase):
         self.strict_cloud = openstack.connection.Connection(
             config=self.cloud_config,
             strict=True)
+        self.addCleanup(self.cloud.task_manager.stop)
+        self.addCleanup(self.strict_cloud.task_manager.stop)
 
         # FIXME(notmorgan): Convert the uri_registry, discovery.json, and
         # use of keystone_v3/v2 to a proper fixtures.Fixture. For now this
@@ -689,7 +691,8 @@ class IronicTestCase(TestCase):
         self.uuid = str(uuid.uuid4())
         self.name = self.getUniqueString('name')
 
-    def get_mock_url(self, resource=None, append=None, qs_elements=None):
-        return super(IronicTestCase, self).get_mock_url(
-            service_type='baremetal', interface='public', resource=resource,
-            append=append, base_url_append='v1', qs_elements=qs_elements)
+    def get_mock_url(self, **kwargs):
+        kwargs.setdefault('service_type', 'baremetal')
+        kwargs.setdefault('interface', 'public')
+        kwargs.setdefault('base_url_append', 'v1')
+        return super(IronicTestCase, self).get_mock_url(**kwargs)
