@@ -24,7 +24,6 @@ import yaml
 from openstack import config
 from openstack.config import cloud_region
 from openstack.config import defaults
-from openstack.config import loader
 from openstack import exceptions
 from openstack.tests.unit.config import base
 
@@ -47,7 +46,7 @@ class TestConfig(base.TestCase):
             cloud for cloud in base.USER_CONF['clouds'].keys()
         ] + ['_test_cloud_regions']
         configured_clouds = [cloud.name for cloud in clouds]
-        self.assertItemsEqual(user_clouds, configured_clouds)
+        self.assertCountEqual(user_clouds, configured_clouds)
 
     def test_get_all_clouds(self):
         # Ensure the alias is in place
@@ -61,7 +60,7 @@ class TestConfig(base.TestCase):
             cloud for cloud in base.USER_CONF['clouds'].keys()
         ] + ['_test_cloud_regions']
         configured_clouds = [cloud.name for cloud in clouds]
-        self.assertItemsEqual(user_clouds, configured_clouds)
+        self.assertCountEqual(user_clouds, configured_clouds)
 
     def test_get_one(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -105,22 +104,6 @@ class TestConfig(base.TestCase):
             defaults._defaults['auth_type'],
             cc.auth_type,
         )
-        self.assertEqual(
-            defaults._defaults['identity_api_version'],
-            cc.identity_api_version,
-        )
-
-    def test_get_one_auth_override_defaults(self):
-        default_options = {'compute_api_version': '4'}
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   override_defaults=default_options)
-        cc = c.get_one(cloud='_test-cloud_', auth={'username': 'user'})
-        self.assertEqual('user', cc.auth['username'])
-        self.assertEqual('4', cc.compute_api_version)
-        self.assertEqual(
-            defaults._defaults['identity_api_version'],
-            cc.identity_api_version,
-        )
 
     def test_get_one_with_config_files(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -400,7 +383,7 @@ class TestConfig(base.TestCase):
 
 
 class TestExcludedFormattedConfigValue(base.TestCase):
-    # verify LaunchPad bug #1635696
+    # verify https://storyboard.openstack.org/#!/story/1635696
     #
     # get_one_cloud() and get_one_cloud_osc() iterate over config
     # values and try to expand any variables in those values by
@@ -1020,13 +1003,6 @@ class TestConfigDefault(base.TestCase):
         self._assert_cloud_details(cc)
         self.assertEqual('password', cc.auth_type)
 
-    def test_set_default_before_init(self):
-        loader.set_default('identity_api_version', '4')
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   vendor_files=[self.vendor_yaml])
-        cc = c.get_one(cloud='_test-cloud_', argparse=None)
-        self.assertEqual('4', cc.identity_api_version)
-
 
 class TestBackwardsCompatibility(base.TestCase):
 
