@@ -45,10 +45,9 @@ class TestFloatingIP(base.TestCase):
         self.assertEqual('floatingip', sot.resource_key)
         self.assertEqual('floatingips', sot.resources_key)
         self.assertEqual('/floatingips', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -76,6 +75,7 @@ class TestFloatingIP(base.TestCase):
     def test_find_available(self):
         mock_session = mock.Mock(spec=adapter.Adapter)
         mock_session.get_filter = mock.Mock(return_value={})
+        mock_session.default_microversion = None
         data = {'id': 'one', 'floating_ip_address': '10.0.0.1'}
         fake_response = mock.Mock()
         body = {floating_ip.FloatingIP.resources_key: [data]}
@@ -89,10 +89,12 @@ class TestFloatingIP(base.TestCase):
         mock_session.get.assert_called_with(
             floating_ip.FloatingIP.base_path,
             headers={'Accept': 'application/json'},
-            params={})
+            params={},
+            microversion=None)
 
     def test_find_available_nada(self):
         mock_session = mock.Mock(spec=adapter.Adapter)
+        mock_session.default_microversion = None
         fake_response = mock.Mock()
         body = {floating_ip.FloatingIP.resources_key: []}
         fake_response.json = mock.Mock(return_value=body)
