@@ -14,6 +14,7 @@ import itertools
 
 from keystoneauth1 import adapter
 import mock
+import munch
 import requests
 import six
 
@@ -65,7 +66,7 @@ class TestComponent(base.TestCase):
 
         # Test that we short-circuit everything when given no instance.
         result = sot.__get__(None, None)
-        self.assertIsNone(result)
+        self.assertIs(sot, result)
 
     # NOTE: Some tests will use a default=1 setting when testing result
     # values that should be None because the default-for-default is also None.
@@ -311,7 +312,7 @@ class TestComponentManager(base.TestCase):
     def test_iter(self):
         attrs = {"key": "value"}
         sot = resource._ComponentManager(attributes=attrs)
-        self.assertItemsEqual(iter(attrs), sot.__iter__())
+        self.assertCountEqual(iter(attrs), sot.__iter__())
 
     def test_len(self):
         attrs = {"key": "value"}
@@ -400,17 +401,22 @@ class TestResource(base.TestCase):
         body = {"body": 1}
         header = {"header": 2, "Location": "somewhere"}
         uri = {"uri": 3}
-        everything = dict(itertools.chain(body.items(), header.items(),
-                                          uri.items()))
+        computed = {"computed": 4}
+        everything = dict(itertools.chain(
+            body.items(),
+            header.items(),
+            uri.items(),
+            computed.items(),
+        ))
 
         mock_collect = mock.Mock()
-        mock_collect.return_value = body, header, uri
+        mock_collect.return_value = body, header, uri, computed
 
         with mock.patch.object(resource.Resource,
                                "_collect_attrs", mock_collect):
             sot = resource.Resource(_synchronized=False, **everything)
             mock_collect.assert_called_once_with(everything)
-        self.assertEqual("somewhere", sot.location)
+        self.assertIsNone(sot.location)
 
         self.assertIsInstance(sot._body, resource._ComponentManager)
         self.assertEqual(body, sot._body.dirty)
@@ -420,18 +426,19 @@ class TestResource(base.TestCase):
         self.assertEqual(uri, sot._uri.dirty)
 
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertFalse(sot.allow_list)
         self.assertFalse(sot.allow_head)
-        self.assertEqual('PUT', sot.update_method)
+        self.assertEqual('PUT', sot.commit_method)
         self.assertEqual('POST', sot.create_method)
 
     def test_repr(self):
         a = {"a": 1}
         b = {"b": 2}
         c = {"c": 3}
+        d = {"d": 4}
 
         class Test(resource.Resource):
             def __init__(self):
@@ -447,6 +454,10 @@ class TestResource(base.TestCase):
                 self._uri.attributes.items = mock.Mock(
                     return_value=c.items())
 
+                self._computed = mock.Mock()
+                self._computed.attributes.items = mock.Mock(
+                    return_value=d.items())
+
         the_repr = repr(Test())
 
         # Don't test the arguments all together since the dictionary order
@@ -455,6 +466,7 @@ class TestResource(base.TestCase):
         self.assertIn("a=1", the_repr)
         self.assertIn("b=2", the_repr)
         self.assertIn("c=3", the_repr)
+        self.assertIn("d=4", the_repr)
 
     def test_equality(self):
         class Example(resource.Resource):
@@ -476,11 +488,14 @@ class TestResource(base.TestCase):
         body = "body"
         header = "header"
         uri = "uri"
+        computed = "computed"
 
-        sot._collect_attrs = mock.Mock(return_value=(body, header, uri))
+        sot._collect_attrs = mock.Mock(
+            return_value=(body, header, uri, computed))
         sot._body.update = mock.Mock()
         sot._header.update = mock.Mock()
         sot._uri.update = mock.Mock()
+        sot._computed.update = mock.Mock()
 
         args = {"arg": 1}
         sot._update(**args)
@@ -489,19 +504,7 @@ class TestResource(base.TestCase):
         sot._body.update.assert_called_once_with(body)
         sot._header.update.assert_called_once_with(header)
         sot._uri.update.assert_called_once_with(uri)
-
-    def test__collect_attrs(self):
-        sot = resource.Resource()
-
-        expected_attrs = ["body", "header", "uri"]
-
-        sot._consume_attrs = mock.Mock()
-        sot._consume_attrs.side_effect = expected_attrs
-
-        # It'll get passed an empty dict at the least.
-        actual_attrs = sot._collect_attrs(dict())
-
-        self.assertItemsEqual(expected_attrs, actual_attrs)
+        sot._computed.update.assert_called_with(computed)
 
     def test__consume_attrs(self):
         serverside_key1 = "someKey1"
@@ -537,7 +540,7 @@ class TestResource(base.TestCase):
         # Check that even on an empty class, we get the expected
         # built-in attributes.
 
-        self.assertIn("location", resource.Resource._header_mapping())
+        self.assertIn("location", resource.Resource._computed_mapping())
         self.assertIn("name", resource.Resource._body_mapping())
         self.assertIn("id", resource.Resource._body_mapping())
 
@@ -694,7 +697,8 @@ class TestResource(base.TestCase):
         expected = {
             'id': 'FAKE_ID',
             'name': None,
-            'bar': None
+            'bar': None,
+            'location': None,
         }
         self.assertEqual(expected, res.to_dict(headers=False))
 
@@ -743,10 +747,13 @@ class TestResource(base.TestCase):
 
         res = Test(id='FAKE_ID')
 
-        err = self.assertRaises(ValueError,
-                                res.to_dict, body=False, headers=False)
-        self.assertEqual('At least one of `body` or `headers` must be True',
-                         six.text_type(err))
+        err = self.assertRaises(
+            ValueError,
+            res.to_dict,
+            body=False, headers=False, computed=False)
+        self.assertEqual(
+            'At least one of `body`, `headers` or `computed` must be True',
+            six.text_type(err))
 
     def test_to_dict_with_mro_no_override(self):
 
@@ -791,6 +798,28 @@ class TestResource(base.TestCase):
         self.assertNotIn("attr", sot._body.dirty)
         self.assertEqual(value, sot.attr)
 
+    def test_from_munch_new(self):
+        class Test(resource.Resource):
+            attr = resource.Body("body_attr")
+
+        value = "value"
+        orig = munch.Munch(body_attr=value)
+        sot = Test._from_munch(orig, synchronized=False)
+
+        self.assertIn("body_attr", sot._body.dirty)
+        self.assertEqual(value, sot.attr)
+
+    def test_from_munch_existing(self):
+        class Test(resource.Resource):
+            attr = resource.Body("body_attr")
+
+        value = "value"
+        orig = munch.Munch(body_attr=value)
+        sot = Test._from_munch(orig)
+
+        self.assertNotIn("body_attr", sot._body.dirty)
+        self.assertEqual(value, sot.attr)
+
     def test__prepare_request_with_id(self):
         class Test(resource.Resource):
             base_path = "/something"
@@ -835,6 +864,39 @@ class TestResource(base.TestCase):
         self.assertEqual({key: {"x": body_value}}, result.body)
         self.assertEqual({"y": header_value}, result.headers)
 
+    def test__prepare_request_with_patch(self):
+        class Test(resource.Resource):
+            commit_jsonpatch = True
+            base_path = "/something"
+            x = resource.Body("x")
+            y = resource.Body("y")
+
+        the_id = "id"
+        sot = Test.existing(id=the_id, x=1, y=2)
+        sot.x = 3
+
+        result = sot._prepare_request(requires_id=True, patch=True)
+
+        self.assertEqual("something/id", result.url)
+        self.assertEqual([{'op': 'replace', 'path': '/x', 'value': 3}],
+                         result.body)
+
+    def test__prepare_request_with_patch_not_synchronized(self):
+        class Test(resource.Resource):
+            commit_jsonpatch = True
+            base_path = "/something"
+            x = resource.Body("x")
+            y = resource.Body("y")
+
+        the_id = "id"
+        sot = Test.new(id=the_id, x=1)
+
+        result = sot._prepare_request(requires_id=True, patch=True)
+
+        self.assertEqual("something/id", result.url)
+        self.assertEqual([{'op': 'add', 'path': '/x', 'value': 1}],
+                         result.body)
+
     def test__translate_response_no_body(self):
         class Test(resource.Resource):
             attr = resource.Header("attr")
@@ -886,8 +948,8 @@ class TestResource(base.TestCase):
     def test_cant_do_anything(self):
         class Test(resource.Resource):
             allow_create = False
-            allow_get = False
-            allow_update = False
+            allow_fetch = False
+            allow_commit = False
             allow_delete = False
             allow_head = False
             allow_list = False
@@ -897,7 +959,7 @@ class TestResource(base.TestCase):
         # The first argument to all of these operations is the session,
         # but we raise before we get to it so just pass anything in.
         self.assertRaises(exceptions.MethodNotSupported, sot.create, "")
-        self.assertRaises(exceptions.MethodNotSupported, sot.get, "")
+        self.assertRaises(exceptions.MethodNotSupported, sot.fetch, "")
         self.assertRaises(exceptions.MethodNotSupported, sot.delete, "")
         self.assertRaises(exceptions.MethodNotSupported, sot.head, "")
 
@@ -910,7 +972,7 @@ class TestResource(base.TestCase):
         # if the call can be made, so fake a dirty list.
         sot._body = mock.Mock()
         sot._body.dirty = mock.Mock(return_value={"x": "y"})
-        self.assertRaises(exceptions.MethodNotSupported, sot.update, "")
+        self.assertRaises(exceptions.MethodNotSupported, sot.commit, "")
 
 
 class TestResourceActions(base.TestCase):
@@ -926,9 +988,9 @@ class TestResourceActions(base.TestCase):
             base_path = self.base_path
             resources_key = 'resources'
             allow_create = True
-            allow_get = True
+            allow_fetch = True
             allow_head = True
-            allow_update = True
+            allow_commit = True
             allow_delete = True
             allow_list = True
 
@@ -953,8 +1015,15 @@ class TestResourceActions(base.TestCase):
         self.session.post = mock.Mock(return_value=self.response)
         self.session.delete = mock.Mock(return_value=self.response)
         self.session.head = mock.Mock(return_value=self.response)
+        self.session.default_microversion = None
+        self.session.retriable_status_codes = None
 
-    def _test_create(self, cls, requires_id=False, prepend_key=False):
+        self.endpoint_data = mock.Mock(max_microversion='1.99',
+                                       min_microversion=None)
+        self.session.get_endpoint_data.return_value = self.endpoint_data
+
+    def _test_create(self, cls, requires_id=False, prepend_key=False,
+                     microversion=None):
         id = "id" if requires_id else None
         sot = cls(id=id)
         sot._prepare_request = mock.Mock(return_value=self.request)
@@ -967,12 +1036,15 @@ class TestResourceActions(base.TestCase):
         if requires_id:
             self.session.put.assert_called_once_with(
                 self.request.url,
-                json=self.request.body, headers=self.request.headers)
+                json=self.request.body, headers=self.request.headers,
+                microversion=microversion)
         else:
             self.session.post.assert_called_once_with(
                 self.request.url,
-                json=self.request.body, headers=self.request.headers)
+                json=self.request.body, headers=self.request.headers,
+                microversion=microversion)
 
+        self.assertEqual(sot.microversion, microversion)
         sot._translate_response.assert_called_once_with(self.response)
         self.assertEqual(result, sot)
 
@@ -985,6 +1057,17 @@ class TestResourceActions(base.TestCase):
 
         self._test_create(Test, requires_id=True, prepend_key=True)
 
+    def test_put_create_with_microversion(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_create = True
+            create_method = 'PUT'
+            _max_microversion = '1.42'
+
+        self._test_create(Test, requires_id=True, prepend_key=True,
+                          microversion='1.42')
+
     def test_post_create(self):
         class Test(resource.Resource):
             service = self.service_name
@@ -994,22 +1077,44 @@ class TestResourceActions(base.TestCase):
 
         self._test_create(Test, requires_id=False, prepend_key=True)
 
-    def test_get(self):
-        result = self.sot.get(self.session)
+    def test_fetch(self):
+        result = self.sot.fetch(self.session)
 
         self.sot._prepare_request.assert_called_once_with(requires_id=True)
         self.session.get.assert_called_once_with(
-            self.request.url,)
+            self.request.url, microversion=None)
 
+        self.assertIsNone(self.sot.microversion)
         self.sot._translate_response.assert_called_once_with(self.response)
         self.assertEqual(result, self.sot)
 
-    def test_get_not_requires_id(self):
-        result = self.sot.get(self.session, False)
+    def test_fetch_with_microversion(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_fetch = True
+            _max_microversion = '1.42'
+
+        sot = Test(id='id')
+        sot._prepare_request = mock.Mock(return_value=self.request)
+        sot._translate_response = mock.Mock()
+
+        result = sot.fetch(self.session)
+
+        sot._prepare_request.assert_called_once_with(requires_id=True)
+        self.session.get.assert_called_once_with(
+            self.request.url, microversion='1.42')
+
+        self.assertEqual(sot.microversion, '1.42')
+        sot._translate_response.assert_called_once_with(self.response)
+        self.assertEqual(result, sot)
+
+    def test_fetch_not_requires_id(self):
+        result = self.sot.fetch(self.session, False)
 
         self.sot._prepare_request.assert_called_once_with(requires_id=False)
         self.session.get.assert_called_once_with(
-            self.request.url,)
+            self.request.url, microversion=None)
 
         self.sot._translate_response.assert_called_once_with(self.response)
         self.assertEqual(result, self.sot)
@@ -1020,56 +1125,113 @@ class TestResourceActions(base.TestCase):
         self.sot._prepare_request.assert_called_once_with()
         self.session.head.assert_called_once_with(
             self.request.url,
-            headers={"Accept": ""})
+            headers={"Accept": ""},
+            microversion=None)
 
+        self.assertIsNone(self.sot.microversion)
         self.sot._translate_response.assert_called_once_with(
             self.response, has_body=False)
         self.assertEqual(result, self.sot)
 
-    def _test_update(self, update_method='PUT', prepend_key=True,
-                     has_body=True):
-        self.sot.update_method = update_method
+    def test_head_with_microversion(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_head = True
+            _max_microversion = '1.42'
+
+        sot = Test(id='id')
+        sot._prepare_request = mock.Mock(return_value=self.request)
+        sot._translate_response = mock.Mock()
+
+        result = sot.head(self.session)
+
+        sot._prepare_request.assert_called_once_with()
+        self.session.head.assert_called_once_with(
+            self.request.url,
+            headers={"Accept": ""},
+            microversion='1.42')
+
+        self.assertEqual(sot.microversion, '1.42')
+        sot._translate_response.assert_called_once_with(
+            self.response, has_body=False)
+        self.assertEqual(result, sot)
+
+    def _test_commit(self, commit_method='PUT', prepend_key=True,
+                     has_body=True, microversion=None,
+                     commit_args=None, expected_args=None):
+        self.sot.commit_method = commit_method
 
         # Need to make sot look dirty so we can attempt an update
         self.sot._body = mock.Mock()
         self.sot._body.dirty = mock.Mock(return_value={"x": "y"})
 
-        self.sot.update(self.session, prepend_key=prepend_key,
-                        has_body=has_body)
+        self.sot.commit(self.session, prepend_key=prepend_key,
+                        has_body=has_body, **(commit_args or {}))
 
         self.sot._prepare_request.assert_called_once_with(
             prepend_key=prepend_key)
 
-        if update_method == 'PATCH':
+        if commit_method == 'PATCH':
             self.session.patch.assert_called_once_with(
                 self.request.url,
-                json=self.request.body, headers=self.request.headers)
-        elif update_method == 'POST':
+                json=self.request.body, headers=self.request.headers,
+                microversion=microversion, **(expected_args or {}))
+        elif commit_method == 'POST':
             self.session.post.assert_called_once_with(
                 self.request.url,
-                json=self.request.body, headers=self.request.headers)
-        elif update_method == 'PUT':
+                json=self.request.body, headers=self.request.headers,
+                microversion=microversion, **(expected_args or {}))
+        elif commit_method == 'PUT':
             self.session.put.assert_called_once_with(
                 self.request.url,
-                json=self.request.body, headers=self.request.headers)
+                json=self.request.body, headers=self.request.headers,
+                microversion=microversion, **(expected_args or {}))
 
+        self.assertEqual(self.sot.microversion, microversion)
         self.sot._translate_response.assert_called_once_with(
             self.response, has_body=has_body)
 
-    def test_update_put(self):
-        self._test_update(update_method='PUT', prepend_key=True, has_body=True)
+    def test_commit_put(self):
+        self._test_commit(commit_method='PUT', prepend_key=True, has_body=True)
 
-    def test_update_patch(self):
-        self._test_update(
-            update_method='PATCH', prepend_key=False, has_body=False)
+    def test_commit_patch(self):
+        self._test_commit(
+            commit_method='PATCH', prepend_key=False, has_body=False)
 
-    def test_update_not_dirty(self):
+    def test_commit_patch_retry_on_conflict(self):
+        self._test_commit(
+            commit_method='PATCH',
+            commit_args={'retry_on_conflict': True},
+            expected_args={'retriable_status_codes': {409}})
+
+    def test_commit_put_retry_on_conflict(self):
+        self._test_commit(
+            commit_method='PUT',
+            commit_args={'retry_on_conflict': True},
+            expected_args={'retriable_status_codes': {409}})
+
+    def test_commit_patch_no_retry_on_conflict(self):
+        self.session.retriable_status_codes = {409, 503}
+        self._test_commit(
+            commit_method='PATCH',
+            commit_args={'retry_on_conflict': False},
+            expected_args={'retriable_status_codes': {503}})
+
+    def test_commit_put_no_retry_on_conflict(self):
+        self.session.retriable_status_codes = {409, 503}
+        self._test_commit(
+            commit_method='PATCH',
+            commit_args={'retry_on_conflict': False},
+            expected_args={'retriable_status_codes': {503}})
+
+    def test_commit_not_dirty(self):
         self.sot._body = mock.Mock()
         self.sot._body.dirty = dict()
         self.sot._header = mock.Mock()
         self.sot._header.dirty = dict()
 
-        self.sot.update(self.session)
+        self.sot.commit(self.session)
 
         self.session.put.assert_not_called()
 
@@ -1079,12 +1241,36 @@ class TestResourceActions(base.TestCase):
         self.sot._prepare_request.assert_called_once_with()
         self.session.delete.assert_called_once_with(
             self.request.url,
-            headers={"Accept": ""})
+            headers={"Accept": ""},
+            microversion=None)
 
         self.sot._translate_response.assert_called_once_with(
             self.response, has_body=False)
         self.assertEqual(result, self.sot)
 
+    def test_delete_with_microversion(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_delete = True
+            _max_microversion = '1.42'
+
+        sot = Test(id='id')
+        sot._prepare_request = mock.Mock(return_value=self.request)
+        sot._translate_response = mock.Mock()
+
+        result = sot.delete(self.session)
+
+        sot._prepare_request.assert_called_once_with()
+        self.session.delete.assert_called_once_with(
+            self.request.url,
+            headers={"Accept": ""},
+            microversion='1.42')
+
+        sot._translate_response.assert_called_once_with(
+            self.response, has_body=False)
+        self.assertEqual(result, sot)
+
     # NOTE: As list returns a generator, testing it requires consuming
     # the generator. Wrap calls to self.sot.list in a `list`
     # and then test the results as a list of responses.
@@ -1100,7 +1286,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_once_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         self.assertEqual([], result)
 
@@ -1136,7 +1323,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_once_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         self.assertEqual(1, len(results))
         self.assertEqual(id_value, results[0].id)
@@ -1162,7 +1350,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_once_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         self.assertEqual(1, len(results))
         self.assertEqual(id_value, results[0].id)
@@ -1196,11 +1385,13 @@ class TestResourceActions(base.TestCase):
         self.assertEqual(ids[1], results[1].id)
         self.assertEqual(
             mock.call('base_path',
-                      headers={'Accept': 'application/json'}, params={}),
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion=None),
             self.session.get.mock_calls[0])
         self.assertEqual(
             mock.call('https://example.com/next-url',
-                      headers={'Accept': 'application/json'}, params={}),
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion=None),
             self.session.get.mock_calls[1])
         self.assertEqual(2, len(self.session.get.call_args_list))
         self.assertIsInstance(results[0], self.test_class)
@@ -1230,15 +1421,64 @@ class TestResourceActions(base.TestCase):
         self.assertEqual(ids[1], results[1].id)
         self.assertEqual(
             mock.call('base_path',
-                      headers={'Accept': 'application/json'}, params={}),
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion=None),
             self.session.get.mock_calls[0])
         self.assertEqual(
             mock.call('https://example.com/next-url',
-                      headers={'Accept': 'application/json'}, params={}),
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion=None),
             self.session.get.mock_calls[2])
         self.assertEqual(2, len(self.session.get.call_args_list))
         self.assertIsInstance(results[0], self.test_class)
 
+    def test_list_response_paginated_with_microversions(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            resources_key = 'resources'
+            allow_list = True
+            _max_microversion = '1.42'
+
+        ids = [1, 2]
+        mock_response = mock.Mock()
+        mock_response.status_code = 200
+        mock_response.links = {}
+        mock_response.json.return_value = {
+            "resources": [{"id": ids[0]}],
+            "resources_links": [{
+                "href": "https://example.com/next-url",
+                "rel": "next",
+            }]
+        }
+        mock_response2 = mock.Mock()
+        mock_response2.status_code = 200
+        mock_response2.links = {}
+        mock_response2.json.return_value = {
+            "resources": [{"id": ids[1]}],
+        }
+
+        self.session.get.side_effect = [mock_response, mock_response2]
+
+        results = list(Test.list(self.session, paginated=True))
+
+        self.assertEqual(2, len(results))
+        self.assertEqual(ids[0], results[0].id)
+        self.assertEqual(ids[1], results[1].id)
+        self.assertEqual(
+            mock.call('base_path',
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion='1.42'),
+            self.session.get.mock_calls[0])
+        self.assertEqual(
+            mock.call('https://example.com/next-url',
+                      headers={'Accept': 'application/json'}, params={},
+                      microversion='1.42'),
+            self.session.get.mock_calls[1])
+        self.assertEqual(2, len(self.session.get.call_args_list))
+        self.assertIsInstance(results[0], Test)
+        self.assertEqual('1.42', results[0].microversion)
+
     def test_list_multi_page_response_not_paginated(self):
         ids = [1, 2]
         mock_response = mock.Mock()
@@ -1430,20 +1670,23 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         result1 = next(results)
         self.assertEqual(result1.id, ids[1])
         self.session.get.assert_called_with(
             'https://example.com/next-url',
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         self.assertRaises(StopIteration, next, results)
         self.session.get.assert_called_with(
             'https://example.com/next-url',
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
     def test_list_multi_page_no_early_termination(self):
         # This tests verifies that multipages are not early terminated.
@@ -1485,7 +1728,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={"limit": 3})
+            params={"limit": 3},
+            microversion=None)
 
         # Second page contains another two items
         result2 = next(results)
@@ -1495,7 +1739,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={"limit": 3, "marker": 2})
+            params={"limit": 3, "marker": 2},
+            microversion=None)
 
         # Ensure we're done after those four items
         self.assertRaises(StopIteration, next, results)
@@ -1504,7 +1749,8 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={"limit": 3, "marker": 4})
+            params={"limit": 3, "marker": 4},
+            microversion=None)
 
         # Ensure we made three calls to get this done
         self.assertEqual(3, len(self.session.get.call_args_list))
@@ -1541,14 +1787,16 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={"limit": 2})
+            params={"limit": 2},
+            microversion=None)
 
         result2 = next(results)
         self.assertEqual(result2.id, ids[2])
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={'limit': 2, 'marker': 2})
+            params={'limit': 2, 'marker': 2},
+            microversion=None)
 
         # Ensure we're done after those three items
         self.assertRaises(StopIteration, next, results)
@@ -1589,14 +1837,16 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         result2 = next(results)
         self.assertEqual(result2.id, ids[2])
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={'marker': 2})
+            params={'marker': 2},
+            microversion=None)
 
         # Ensure we're done after those three items
         self.assertRaises(StopIteration, next, results)
@@ -1635,14 +1885,16 @@ class TestResourceActions(base.TestCase):
         self.session.get.assert_called_with(
             self.base_path,
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         result2 = next(results)
         self.assertEqual(result2.id, ids[2])
         self.session.get.assert_called_with(
             'https://example.com/next-url',
             headers={"Accept": "application/json"},
-            params={})
+            params={},
+            microversion=None)
 
         # Ensure we're done after those three items
         self.assertRaises(StopIteration, next, results)
@@ -1664,7 +1916,7 @@ class TestResourceFind(base.TestCase):
             def existing(cls, **kwargs):
                 response = mock.Mock()
                 response.status_code = 404
-                raise exceptions.NotFoundException(
+                raise exceptions.ResourceNotFound(
                     'Not Found', response=response)
 
             @classmethod
@@ -1694,7 +1946,7 @@ class TestResourceFind(base.TestCase):
             @classmethod
             def existing(cls, **kwargs):
                 mock_match = mock.Mock()
-                mock_match.get.return_value = value
+                mock_match.fetch.return_value = value
                 return mock_match
 
         result = Test.find("session", "name")
@@ -1805,11 +2057,11 @@ class TestWaitForStatus(base.TestCase):
         assert not kwargs, 'Unexpected keyword arguments: %s' % kwargs
         resources = []
         for status in statuses:
-            res = mock.Mock(spec=['id', 'get', attribute])
+            res = mock.Mock(spec=['id', 'fetch', attribute])
             setattr(res, attribute, status)
             resources.append(res)
         for index, res in enumerate(resources[:-1]):
-            res.get.return_value = resources[index + 1]
+            res.fetch.return_value = resources[index + 1]
         return resources
 
     def test_status_match(self):
@@ -1904,9 +2156,9 @@ class TestWaitForDelete(base.TestCase):
         response.headers = {}
         response.status_code = 404
         res = mock.Mock()
-        res.get.side_effect = [
+        res.fetch.side_effect = [
             None, None,
-            exceptions.NotFoundException('Not Found', response)]
+            exceptions.ResourceNotFound('Not Found', response)]
 
         result = resource.wait_for_delete("session", res, 1, 3)
 
@@ -1915,9 +2167,51 @@ class TestWaitForDelete(base.TestCase):
     def test_timeout(self):
         res = mock.Mock()
         res.status = 'ACTIVE'
-        res.get.return_value = res
+        res.fetch.return_value = res
 
         self.assertRaises(
             exceptions.ResourceTimeout,
             resource.wait_for_delete,
             "session", res, 0.1, 0.3)
+
+
+@mock.patch.object(resource.Resource, '_get_microversion_for', autospec=True)
+class TestAssertMicroversionFor(base.TestCase):
+    session = mock.Mock()
+    res = resource.Resource()
+
+    def test_compatible(self, mock_get_ver):
+        mock_get_ver.return_value = '1.42'
+
+        self.assertEqual(
+            '1.42',
+            self.res._assert_microversion_for(self.session, 'fetch', '1.6'))
+        mock_get_ver.assert_called_once_with(self.res, self.session, 'fetch')
+
+    def test_incompatible(self, mock_get_ver):
+        mock_get_ver.return_value = '1.1'
+
+        self.assertRaisesRegex(exceptions.NotSupported,
+                               '1.6 is required, but 1.1 will be used',
+                               self.res._assert_microversion_for,
+                               self.session, 'fetch', '1.6')
+        mock_get_ver.assert_called_once_with(self.res, self.session, 'fetch')
+
+    def test_custom_message(self, mock_get_ver):
+        mock_get_ver.return_value = '1.1'
+
+        self.assertRaisesRegex(exceptions.NotSupported,
+                               'boom.*1.6 is required, but 1.1 will be used',
+                               self.res._assert_microversion_for,
+                               self.session, 'fetch', '1.6',
+                               error_message='boom')
+        mock_get_ver.assert_called_once_with(self.res, self.session, 'fetch')
+
+    def test_none(self, mock_get_ver):
+        mock_get_ver.return_value = None
+
+        self.assertRaisesRegex(exceptions.NotSupported,
+                               '1.6 is required, but the default version',
+                               self.res._assert_microversion_for,
+                               self.session, 'fetch', '1.6')
+        mock_get_ver.assert_called_once_with(self.res, self.session, 'fetch')
